@@ -13,6 +13,7 @@ use crate::module::{FlatpakBuildOptions, FlatpakModule, FlatpakModuleItem};
 #[derive(Deserialize)]
 #[derive(Serialize)]
 #[derive(Debug)]
+#[derive(Hash)]
 #[derive(Default)]
 #[serde(rename_all = "kebab-case")]
 #[serde(default)]
@@ -220,26 +221,23 @@ impl FlatpakApplication {
             return Err(format!("{} is not a file.", path));
         }
 
-        if FlatpakApplication::file_path_matches(&file_path.to_str().unwrap()) {
-            let manifest_content = match fs::read_to_string(file_path) {
-                Ok(content) => content,
-                Err(e) => {
-                    return Err(format!(
-                        "Could not read file {}: {}!",
-                        file_path.to_str().unwrap(),
-                        e
-                    ))
-                }
-            };
+        let manifest_format = match FlatpakManifestFormat::from_path(&path) {
+            Some(f) => f,
+            None => return Err(format!("{} is not a Flatpak application manifest.", path)),
+        };
 
-            let manifest = match FlatpakApplication::parse(&path, &manifest_content) {
-                Ok(m) => m,
-                Err(e) => return Err(format!("Failed to parse Flatpak manifest at {}: {}", path, e)),
-            };
-
-            return Ok(manifest);
-        } else {
-            return Err(format!("{} is not a Flatpak manifest.", path));
+        let manifest_content = match fs::read_to_string(file_path) {
+            Ok(content) => content,
+            Err(e) => {
+                return Err(format!("Could not read application manifest at {}: {}", &path, e));
+            }
+        };
+        match FlatpakApplication::parse(manifest_format, &manifest_content) {
+            Ok(m) => Ok(m),
+            Err(e) => Err(format!(
+                "Failed to parse Flatpak application manifest at {}: {}",
+                path, e
+            )),
         }
     }
 
@@ -251,27 +249,31 @@ impl FlatpakApplication {
         crate::filename::is_reverse_dns(&path)
     }
 
-    pub fn parse(manifest_path: &str, manifest_content: &str) -> Result<FlatpakApplication, String> {
-        let mut flatpak_manifest: FlatpakApplication = FlatpakApplication::default();
-
-        if crate::filename::is_yaml(&manifest_path) {
-            flatpak_manifest = match serde_yaml::from_str(&manifest_content) {
+    pub fn parse(format: FlatpakManifestFormat, manifest_content: &str) -> Result<FlatpakApplication, String> {
+        let mut flatpak_manifest: FlatpakApplication = match &format {
+            FlatpakManifestFormat::YAML => match serde_yaml::from_str(&manifest_content) {
                 Ok(m) => m,
                 Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+                    return Err(format!(
+                        "Failed to parse the Flatpak application manifest: {}.",
+                        e
+                    ));
                 }
-            };
-            flatpak_manifest.format = FlatpakManifestFormat::YAML;
-        } else if crate::filename::is_json(&manifest_path) {
-            let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
-            flatpak_manifest = match serde_json::from_str(&json_content_without_comments) {
-                Ok(m) => m,
-                Err(e) => {
-                    return Err(format!("Failed to parse the Flatpak manifest: {}.", e));
+            },
+            FlatpakManifestFormat::JSON => {
+                let json_content_without_comments = crate::utils::remove_comments_from_json(manifest_content);
+                match serde_json::from_str(&json_content_without_comments) {
+                    Ok(m) => m,
+                    Err(e) => {
+                        return Err(format!(
+                            "Failed to parse the Flatpak application manifest: {}.",
+                            e
+                        ));
+                    }
                 }
-            };
-            flatpak_manifest.format = FlatpakManifestFormat::JSON;
-        }
+            }
+        };
+        flatpak_manifest.format = format;
 
         // From https://docs.flatpak.org/en/latest/manifests.html#basic-properties:
         // Each manifest file should specify basic information about the application that is to be built,
@@ -382,9 +384,10 @@ impl FlatpakApplication {
 #[derive(Serialize)]
 #[derive(Debug)]
 #[derive(Default)]
+#[derive(Hash)]
 #[serde(rename_all = "kebab-case")]
 #[serde(default)]
-/// Extension define extension points in the app/runtime that can be implemented by extensions,
+/// Extensions define extension points in the app/runtime that can be implemented by extensions,
 /// supplying extra files which are available during runtime.
 ///
 /// Additionally the standard flatpak extension properties are supported, and put
@@ -473,92 +476,83 @@ mod tests {
     #[test]
     #[should_panic]
     pub fn test_parse_invalid_yaml() {
-        FlatpakApplication::parse("manifest.yaml", "----------------------------").unwrap();
+        FlatpakApplication::parse(FlatpakManifestFormat::YAML, "----------------------------").unwrap();
     }
 
     #[test]
     pub fn test_parse_missing_fields() {
-        assert!(FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
+        let application_manifest = r###"
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
-            "###
-        )
-        .is_err());
+            command: flatpak-rs
+        "###;
+        assert!(FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest).is_err());
     }
 
     #[test]
     pub fn test_parse() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
+        let application_manifest = r###"
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
                 sources:
                   -
                     type: git
-                    url: https://github.com/louib/flatpak-review.git
+                    url: https://github.com/louib/flatpak-rs.git
                     branch: master
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "net.louib.flatpak-rs");
             }
         }
     }
 
     #[test]
-    pub fn test_parse_shared_modules() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
+    pub fn test_parse_external_modules() {
+        let application_manifest = r###"
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
                 sources:
                   -
                     type: git
-                    url: https://github.com/louib/flatpak-review.git
+                    url: https://github.com/louib/flatpak-rs.git
                     branch: master
               -
                 "shared-modules/linux-audio/lv2.json"
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "net.louib.flatpak-rs");
             }
         }
     }
 
     #[test]
     pub fn test_parse_add_extensions() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
+        let application_manifest = r###"
             app-id: net.pcsx2.PCSX2
             runtime: org.freedesktop.Platform
             runtime-version: "19.08"
@@ -585,55 +579,51 @@ mod tests {
                     merge-dirs: "vulkan/icd.d;glvnd/egl_vendor.d"
                     download-if: "active-gl-driver"
                     enable-if: "active-gl-driver"
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.pcsx2.PCSX2");
-                assert_eq!(manifest.add_extensions.len(), 3);
+            Ok(app) => {
+                assert_eq!(app.get_id(), "net.pcsx2.PCSX2");
+                assert_eq!(app.add_extensions.len(), 3);
             }
         }
     }
 
     #[test]
     pub fn test_parse_string_source() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
+        let application_manifest = r###"
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
-                name: "flatpak-review"
+                name: "flatpak-rs"
                 buildsystem: simple
                 cleanup: [ "*" ]
                 config-opts: []
                 sources:
                   -
                     "shared-modules/linux-audio/lv2.json"
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "net.louib.flatpak-rs");
             }
         }
     }
 
     #[test]
     pub fn test_parse_source_without_type() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
+        let application_manifest = r###"
+            app-id: net.louib.flatpak-rs
             runtime: org.gnome.Platform
             runtime-version: "3.36"
             sdk: org.gnome.Sdk
-            command: flatpak-review
+            command: flatpak-rs
             tags: ["nightly"]
             modules:
               -
@@ -646,92 +636,18 @@ mod tests {
                     url: "https://ftp.gnu.org/gnu/gcc/gcc-7.5.0/gcc-7.5.0.tar.xz"
                     sha256: "b81946e7f01f90528a1f7352ab08cc602b9ccc05d4e44da4bd501c5a189ee661"
 
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
-            }
-        }
-    }
-
-    #[test]
-    pub fn test_parse_build_options() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
-            runtime: org.gnome.Platform
-            runtime-version: "3.36"
-            sdk: org.gnome.Sdk
-            command: flatpak-review
-            tags: ["nightly"]
-            modules:
-              -
-                name: "flatpak-review"
-                buildsystem: simple
-                cleanup: [ "*" ]
-                build-options:
-                   cflags: "-O2 -g"
-                   cxxflags: "-O2 -g"
-                   env:
-                       V: "1"
-                   arch:
-                       x86_64:
-                           cflags: "-O3 -g"
-                config-opts: []
-                sources:
-                  -
-                    "shared-modules/linux-audio/lv2.json"
-            "###,
-        ) {
-            Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
-            }
-        }
-    }
-
-    #[test]
-    pub fn test_parse_script_source() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
-            app-id: net.louib.flatpak-review
-            runtime: org.gnome.Platform
-            runtime-version: "3.36"
-            sdk: org.gnome.Sdk
-            command: flatpak-review
-            tags: ["nightly"]
-            modules:
-              -
-                name: "flatpak-review"
-                buildsystem: simple
-                cleanup: [ "*" ]
-                config-opts: []
-                sources:
-                  -
-                    url: "https://ftp.gnu.org/gnu/gcc/gcc-7.5.0/gcc-7.5.0.tar.xz"
-                    sha256: "b81946e7f01f90528a1f7352ab08cc602b9ccc05d4e44da4bd501c5a189ee661"
-                  -
-                    type: "shell"
-                    commands:
-                      -
-                        sed -i -e 's/\${\${NAME}_BIN}-NOTFOUND/\${NAME}_BIN-NOTFOUND/' cpp/CMakeLists.txt
-            "###,
-        ) {
-            Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "net.louib.flatpak-review");
+            Ok(application) => {
+                assert_eq!(application.get_id(), "net.louib.flatpak-rs");
             }
         }
     }
 
     #[test]
     pub fn test_parse_json() {
-        match FlatpakApplication::parse(
-            "manifest.json",
-            r###"
+        let application_manifest = r###"
             {
                 "app-id": "org.gnome.SoundJuicer",
                 "runtime": "org.gnome.Platform",
@@ -787,20 +703,18 @@ mod tests {
                     }
                 ]
             }
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::JSON, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "org.gnome.SoundJuicer");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "org.gnome.SoundJuicer");
             }
         }
     }
 
     #[test]
     pub fn test_parse_json_with_comments() {
-        match FlatpakApplication::parse(
-            "manifest.json",
-            r###"
+        let application_manifest = r###"
             {
                 "app-id": "org.gnome.SoundJuicer",
                 "runtime": "org.gnome.Platform",
@@ -874,20 +788,18 @@ mod tests {
                     }
                 ]
             }
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::JSON, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "org.gnome.SoundJuicer");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "org.gnome.SoundJuicer");
             }
         }
     }
 
     #[test]
     pub fn test_parse_json_with_multi_line_comments() {
-        match FlatpakApplication::parse(
-            "manifest.json",
-            r###"
+        let application_manifest = r###"
             {
               "app-id": "org.gnome.Lollypop",
               "runtime": "org.gnome.Platform",
@@ -931,51 +843,47 @@ mod tests {
                 }
               ]
             }
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::JSON, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "org.gnome.Lollypop");
+            Ok(app) => {
+                assert_eq!(app.get_id(), "org.gnome.Lollypop");
             }
         }
     }
 
     #[test]
     pub fn test_parse_extension() {
-        match FlatpakApplication::parse(
-            "manifest.json",
-            r###"
-            {
-                "id": "org.freedesktop.Platform.Icontheme.Paper",
-                "branch": "1.0",
-                "runtime": "org.freedesktop.Sdk",
-                "build-extension": true,
-                "sdk": "org.freedesktop.Sdk",
-                "runtime-version": "1.6",
-                "sdk-extensions": [],
-                "separate-locales": false,
-                "cleanup": [ "/share/info", "/share/man" ],
-                "appstream-compose": false,
-                "build-options" : {
-                    "prefix": "/usr/share/runtime"
-                },
-                "modules": []
-            }
-            "###,
-        ) {
+        let extension_manifest = r###"
+        {
+            "id": "org.freedesktop.Platform.Icontheme.Paper",
+            "branch": "1.0",
+            "runtime": "org.freedesktop.Sdk",
+            "build-extension": true,
+            "sdk": "org.freedesktop.Sdk",
+            "runtime-version": "1.6",
+            "sdk-extensions": [],
+            "separate-locales": false,
+            "cleanup": [ "/share/info", "/share/man" ],
+            "appstream-compose": false,
+            "build-options" : {
+                "prefix": "/usr/share/runtime"
+            },
+            "modules": []
+        }
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::JSON, extension_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.id, "org.freedesktop.Platform.Icontheme.Paper");
-                assert_eq!(manifest.get_id(), "org.freedesktop.Platform.Icontheme.Paper");
+            Ok(app) => {
+                assert_eq!(app.id, "org.freedesktop.Platform.Icontheme.Paper");
+                assert_eq!(app.get_id(), "org.freedesktop.Platform.Icontheme.Paper");
             }
         }
     }
 
     #[test]
     pub fn test_parse_recursive_modules() {
-        match FlatpakApplication::parse(
-            "manifest.yaml",
-            r###"
+        let application_manifest = r###"
             app-id: com.georgefb.haruna
             runtime: org.kde.Platform
             runtime-version: '5.15'
@@ -1145,14 +1053,14 @@ mod tests {
                       - type: archive
                         url: 'https://github.com/ytdl-org/youtube-dl/archive/2021.04.26.tar.gz'
                         sha256: 'd80023ab221b3cb89229b632e247035a22c5afaee9a7b3c653bbd702f71c1083'
-            "###,
-        ) {
+        "###;
+        match FlatpakApplication::parse(FlatpakManifestFormat::YAML, application_manifest) {
             Err(e) => std::panic::panic_any(e),
-            Ok(manifest) => {
-                assert_eq!(manifest.app_id, "com.georgefb.haruna");
-                assert_eq!(manifest.get_max_depth(), 3);
-                assert_eq!(manifest.modules.len(), 1);
-                assert_eq!(manifest.get_all_modules_recursively().len(), 12);
+            Ok(app) => {
+                assert_eq!(app.get_id(), "com.georgefb.haruna");
+                assert_eq!(app.get_max_depth(), 3);
+                assert_eq!(app.modules.len(), 1);
+                assert_eq!(app.get_all_modules_recursively().len(), 12);
             }
         }
     }
